<?php

namespace Drupal\webform_prepopulate_js\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformSubmissionInterface;

/**
 * Webform handler to prepopulate fields via JavaScript.
 *
 * @WebformHandler(
 *   id = "webform_prepopulate_js_handler",
 *   label = @Translation("Prepopulate JS Handler"),
 *   category = @Translation("Webform"),
 *   description = @Translation("Adds js script to prepopulate fields from query parameters."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_SINGLE,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_PROCESSED,
 * )
 */
class PrepopulateJsHandler extends WebformHandlerBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Add configuration for fields marked to be prepopulated.
    $webform = $this->getWebform();
    $elements = $webform->getElementsDecoded();

    $options = [];
    foreach ($elements as $key => $element) {
      if (!empty($element['#prepopulate'])) {
        $options[$key] = $element['#title'] ?? $key;
      }
    }

    $form['prepopulate_fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Fields to prepopulate'),
      '#description' => $this->t('Select the fields to prepopulate from the query parameters.'),
      '#options' => $options,
      '#default_value' => $this->configuration['prepopulate_fields'] ?? [],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->configuration['prepopulate_fields'] = array_filter($form_state->getValue('prepopulate_fields'));
  }

  /**
   * Alter the form to add attributes and attach JavaScript.
   *
   * {@inheritdoc}
   */
  public function alterForm(array &$form, FormStateInterface $form_state, WebformSubmissionInterface $webform_submission) {
    parent::alterForm($form, $form_state, $webform_submission);

    // Get selected fields to prepopulate.
    $prepopulate_fields = $this->configuration['prepopulate_fields'] ?? [];
    if (empty($prepopulate_fields)) {
      return;
    }
    // Add "data-prepopulate-js" attribute to selected fields.
    foreach ($prepopulate_fields as $field_key) {
      if (isset($form['elements'][$field_key])) {
        $form['elements'][$field_key]['#attributes']['data-prepopulate-js'] = 'true';
      }
    }

    // Attach the JavaScript library to the form.
    $form['#attached']['library'][] = 'webform_prepopulate_js/webform_prepopulate_js';
  }

}
